import pandas as pd
from dhlab.api.dhlab_api import concordance
from dhlab.text.utils import urnlist, make_link
from IPython.display import HTML


class Concordance(pd.DataFrame):
    @property
    def _constructor(self):
        return Concordance

    @classmethod
    def get_concordances(self, corpus, words, window=20, limit=500):
        res = concordance(urns=urnlist(corpus), words=words, window=window, limit=limit)

        res["link"] = res.urn.apply(make_link)

        res.rename({"conc": "concordance"}, axis=1, inplace=True)

        return Concordance(res)

    def show(self, n=10, style=True):
        if style:
            result = self.sample(min(n, len(self)))[["link", "concordance"]].style
        else:
            result = self.sample(min(n, len(self)))
        return result

    def split_view(self, html=False):
        df = self.concordance.str.split("</?b>", expand=True)
        df.rename(
            {0: "left", 1: "hit", 2: "right", 3: "hit2", 4: "right2"},
            axis=1,
            inplace=True,
        )
        df.index = self.urn

        if html:
            return HTML(df.to_html())
        else:
            return df



import pandas as pd
from dhlab.api.dhlab_api import document_corpus, get_metadata
from dhlab.text.collocations import Collocations
from dhlab.text.concordance import Concordance
from dhlab.text.frequencies import Frequencies
from dhlab.text.utils import urnlist, remove_empty_columns


class Corpus(pd.DataFrame):
    @classmethod
    def build(
        self,
        doctype=None,
        author=None,
        freetext=None,
        fulltext=None,
        from_year=None,
        to_year=None,
        from_timestamp=None,
        to_timestamp=None,
        title=None,
        ddk=None,
        subject=None,
        lang=None,
        limit=10,
        order_by="random",
    ):
        res = document_corpus(
            doctype,
            author,
            freetext,
            fulltext,
            from_year,
            to_year,
            from_timestamp,
            to_timestamp,
            title,
            ddk,
            subject,
            lang,
            limit,
            order_by,
        )
        return remove_empty_columns(Corpus(res))

    def extend_from_identifiers(self, identifiers=None):
        new_corpus = get_metadata(urnlist(identifiers))
        return pd.concat([self, new_corpus], axis=0)

    def get_freqs(self):
        return Frequencies.get_freqs(self)

    def get_concordances(self, words, window=20, limit=500):
        return Concordance.get_concordances(self, words, window=window, limit=limit)

    def get_collocations(
        self,
        #  corpus=None,
        words=None,
        before=10,
        after=10,
        reference=None,
        samplesize=20000,
        alpha=False,
        ignore_caps=False,
    ):
        return Collocations.get_collocations(
            corpus=self,
            words=words,
            before=before,
            after=after,
            reference=reference,
            samplesize=samplesize,
            alpha=alpha,
            ignore_caps=ignore_caps,
        )

    @property
    def _constructor(self):
        return Corpus



import pandas as pd
from dhlab.api.dhlab_api import urn_collocation
from dhlab.text.utils import urnlist


class Collocations(pd.DataFrame):
    @property
    def _constructor(self):
        return Collocations

    @classmethod
    def get_collocations(
        self,
        corpus=None,
        words=None,
        before=10,
        after=10,
        reference=None,
        samplesize=20000,
        alpha=False,
        ignore_caps=False,
    ):
        if isinstance(words, str):
            words = [words]

        res = pd.concat(
            [
                urn_collocation(
                    urns=urnlist(corpus),
                    word=w,
                    before=before,
                    after=after,
                    samplesize=samplesize,
                )
                for w in words
            ]
        )[["counts"]]

        if reference is not None:
            teller = res.counts / res.counts.sum()
            divisor = reference.iloc[:, 0] / reference.iloc[:, 0].sum()
            res["relevance"] = teller / divisor

        return Collocations(res)

    def show(self, sortby="counts", n=20):
        return self.sort_values(by=sortby, ascending=False).head(n)



# code from further down in the code tree

# text
from dhlab.text.corpus import Corpus
from dhlab.text.chunking import Chunks
from dhlab.text.collocations import Collocations
from dhlab.text.concordance import Concordance
from dhlab.text.frequencies import Frequencies
from dhlab.text.wildcard import WildcardWordSearch
from dhlab.utils.display import css
from dhlab.utils.files import download_from_github, get_file_from_github

# ngram
from dhlab.ngram.ngram import Ngram

# api
from dhlab.api.dhlab_api import totals

# metadata
from dhlab.metadata.natbib import metadata_query, metadata_from_urn



import pandas as pd

from dhlab.api.nb_ngram_api import get_ngram


def nb_ngram(
    terms: str,
    corpus: str = "bok",
    smooth: int = 1,
    years: tuple = (1810, 2010),
    mode: str = "relative",
    lang: str = "nob",
):
    """Extract N-gram frequencies from given ``terms`` and ``years``.
    `lang` param is not supported for corpus=`avis` and will be set to None if `avis` is passed.

    :param terms: comma
    :param corpus:
    :param smooth:
    :param years:
    :param mode:
    :param lang:
    :return: A sorted Pandas DataFrame index

    :meta private:
    """
    # Set default lang for 'bok'-corpus
    if corpus == "avis":
        lang = None

    df = ngram_conv(
        get_ngram(terms, corpus=corpus, lang=lang),
        smooth=smooth,
        years=years,
        mode=mode,
    )
    df.index = df.index.astype(int)
    return df.sort_index()


## tar tilbake til original den her virker ikke LGJ
def ngram_conv_old(
    ngrams, smooth: int = 1, years: tuple = (1810, 2013), mode: str = "relative"
):
    """Construct a dataframe with ngram mean frequencies per year over a given time period.

    :param ngrams: TODO: FIll in appropriate type and description.
    :param smooth: Smoothing factor for the graph visualisation.
    :param years: Tuple with start and end years for the time period of interest
    :param mode: Frequency measure. Defaults to 'relative'.
    :return: pandas dataframe with mean values for each year

    :meta private:
    """
    ngc = {}
    # check if relative frequency or absolute frequency is in question
    if mode.startswith("rel") or mode == "y":
        arg = "y"
    else:
        arg = "f"
    for x in ngrams:
        if x and isinstance(x, list):
            ngc[x["key"]] = {
                z["x"]: z[arg]
                for z in x["values"]
                if years[1] >= int(z["x"]) >= years[0]
            }
    return pd.DataFrame(ngc).rolling(window=smooth, win_type="triang").mean()


def ngram_conv(ngrams, smooth=1, years=(1810, 2013), mode="relative"):
    """Construct a dataframe with ngram mean frequencies per year over a given time period.

    :param ngrams: TODO: FIll in appropriate type and description.
    :param smooth: Smoothing factor for the graph visualisation.
    :param years: Tuple with start and end years for the time period of interest
    :param mode: Frequency measure. Defaults to 'relative'.
    :return: pandas dataframe with mean values for each year

    :meta private:
    """
    ngc = {}
    # check if relative frequency or absolute frequency is in question
    if mode.startswith("rel") or mode == "y":
        arg = "y"
    else:
        arg = "f"
    for x in ngrams:
        # check if x is a non empty ngram - empty ngrams are coded as empty lists
        # if x is non emtpy it accepts keys - look at alternative isinstance(x, dict)?
        if x != []:
            ngc[x["key"]] = {
                z["x"]: z[arg]
                for z in x["values"]
                if int(z["x"]) <= int(years[1]) and int(z["x"]) >= int(years[0])
            }

    return pd.DataFrame(ngc).rolling(window=smooth, win_type="triang").mean()



import pandas as pd
from dhlab.api.dhlab_api import get_document_frequencies
from dhlab.text.utils import urnlist


class Frequencies(pd.DataFrame):
    _metadata = ["_title_dct"]

    @classmethod
    def get_freqs(self, corpus, words=None):
        res = get_document_frequencies(urns=urnlist(corpus), words=words)
        self._title_dct = {k: v for k, v in zip(corpus.dhlabid, corpus.title)}
        return Frequencies(res)

    @property
    def _constructor(self):
        return Frequencies

    def sum_freqs(self):
        return self.sum(axis=1).to_frame("frequencies")

    def display_names(self):
        "Display data with record names as column titles."
        return self.rename(self._title_dct, axis=1)



from pandas import DataFrame, Series
import dhlab as dh


def urnlist(corpus):
    """Try to pull out a list of URNs from corpus"""
    if isinstance(corpus, dh.Corpus):
        _urnlist = list(corpus.urn)
    elif isinstance(corpus, DataFrame):
        _urnlist = list(corpus.urn)
    elif isinstance(corpus, list):
        _urnlist = corpus
    elif isinstance(corpus, Series):
        _urnlist = corpus.to_list()
    else:
        _urnlist = []
    return _urnlist


# convert cell to a link
def make_link(row):
    r = "<a target='_blank' href = 'https://urn.nb.no/{x}'>{x}</a>".format(x=str(row))
    return r


def remove_empty_columns(df):
    """
    Remove all columns from a pandas DataFrame that are entirely empty
    or only contain empty strings.

    Parameters:
        df (pd.DataFrame): The input DataFrame.

    Returns:
        pd.DataFrame: The DataFrame without the empty columns.
    """
    return df.drop(columns=df.columns[(df == "").all() | (df.isnull().all())])



from dhlab.api.dhlab_api import wildcard_search


class WildcardWordSearch:
    """
    Find a class of words matching a wildcard string
    """

    def __init__(self, word, factor=2, freq_limit=10, limit=50):
        """:param word: word from a mixture of * and characters
        :factor int: the additional length of words to be returned
        :freq_lim: the frequency of returned words lower limit
        :limit int: number of words returned
        """
        self.words = wildcard_search(
            word, factor=factor, freq_limit=freq_limit, limit=limit
        )
